<?php
namespace App;
/**
 * Open Users - Main Class
 * 
 * Open Users
 * 
 * @copyright SCHLIX Web Inc
 *
 * @license GPLv3
 *
 * @package openusers
 * @version 1.0
 * @author  SCHLIX Web Inc <info@schlix.com>
 * @link    http://www.schlix.com
 */
class OpenUsers extends \App\Users {
    
    public $enabled_providers = array('facebook','google', 'twitter', 'live','linkedin','yahoo');
    

    public function __construct($backend_mode = false) {
        parent::__construct($backend_mode);
    }

        
        public function getEnabledProviderList()
        {
            $result = [];
        
            foreach ($this->enabled_providers as $provider)
            {
                if ($this->getConfig("bool_{$provider}_enabled"))
                {
                    $result[] = ['provider' => $provider, 'url_login' => $this->createFriendlyURL("action=openlogin&provider={$provider}"), 'assoc'=>$this->createFriendlyURL("action=openassoc&provider={$provider}")];
                }
                
            }
            return $result;
        }

        //_______________________________________________________________________________________________________________//
    protected function _internalCreateUser($datavalues, $creategroup = true) {
        global $SystemDB, $SystemConfig;

        $default_group_id = $SystemConfig->get($this->app_name, 'int_default_newuser_group_id');
        if (!$default_group_id)
        {
            $default_group_id = $SystemConfig->get('users', 'int_default_newuser_group_id');
        }
        $SystemDB->query("LOCK TABLES `{$this->table_items}` WRITE");
        $SystemDB->simpleInsertInto($this->table_items, $datavalues);        
        $new_user_id = $SystemDB->getLastInsertID();
        $SystemDB->query("UNLOCK TABLES");
        if ($creategroup) {
            $this->setItemCategory($new_user_id, $default_group_id, true);
        }
    }

	//_______________________________________________________________________________________________________________//			
        /**
         * get User by Open ID
         * @global \SCHLIX\cmsDatabase $SystemDB
         * @param string $provider
         * @param string $user
         * @return boolean
         */
	public function getUserByOpenUserID($provider,$user)
	{
		global $SystemDB;
		
		$provider = alpha_numeric_only($provider); // doesn't hurt to check twice for xss
		if (is_object($user)) $openuserid = sanitize_string($user->identifier); else $openuserid = sanitize_string($user);
		if (empty($user) || empty($provider)) return false;
		$sql =  "SELECT * from {$this->table_items} where {$provider}_user_id = {$openuserid}";
		$result = $SystemDB->getQueryResultSingleRow($sql);
		return $result;
	}
	//_______________________________________________________________________________________________________________//			
	public function getAutoGeneratedUniqueUsername($username, $firstname,$lastname)
	{
		global $SystemDB;
		
		$username = trim($username);
		$full_name = trim ($firstname.' '.$lastname);
		if (empty($username)) $username = convert_into_sef_friendly_title($full_name);
		// if it's still empty... just in case
		if (empty($username)) $username = 'user'.mt_rand(1,999999);
		$theusername = sanitize_string($username);
		$sql = "SELECT {$this->field_username} FROM {$this->table_items} WHERE ({$this->field_username} = {$theusername}) ";
		$results = $SystemDB->getQueryResultArray($sql);
		if ($results)
		{
			$suggested = $username;
			$i = 1;
			while ($this->userNameExists($suggested))
			{
				$suggested = $name.$i;
				$i++;
			}
			return $suggested;
		} else return $username;
	}
	//_______________________________________________________________________________________________________________//	
        /**
         * Create user (facebook, twitter, linkedin, etc)
         * @param string $provider
         * @param array $user
         * @return boolean
         */
	protected function openuser_createUser($provider,$user)
	{
            if (empty($user->email))
                return false;
		$plaintextpasswd=$this->generatePassword(); // auto generate for facebook user in case they wanna login without facebook (who would ??) but the field cannot be empty anyway
		$datavalues = array();
		
                $datavalues['guid'] = new_uuid_v4();
		$datavalues['firstname'] = $user->firstName;
		$datavalues['lastname'] = $user->lastName;
		$datavalues["{$provider}_user_id"] = $user->identifier;
		$datavalues['email_address'] = $user->email;			
		$datavalues['date_created'] = get_current_datetime();
		$datavalues['last_ip_address'] = $_SERVER['REMOTE_ADDR'];
		$datavalues['status'] = 1;
		$datavalues['source'] = $provider;
		$datavalues[$this->field_username] = $this->getAutoGeneratedUniqueUsername($user->username, $datavalues['firstname'], $datavalues['lastname']);
                
                list ($datavalues['salt'], $datavalues[$this->field_password]) = $this->generateNewSaltAndHashedPassword($plaintextpasswd);
                
		$datavalues['additional_info'] = serialize(array("{$provider}_info"=>get_object_vars($user)));
		$this->_internalCreateUser($datavalues,true);
		//
		$last_created_user = $this->getUserInfoByUserName ($datavalues[$this->field_username]);
		$last_created_user_id = $last_created_user['id'];				
		//
		
                $this->sendRegistrationEmailToUserByID($last_created_user_id, 'new-user-registration-success', array('password' => $plaintextpasswd));
		$this->openuser_loginUser($provider,$user);
		return true;
	}
		
	//_______________________________________________________________________________________________________________//			
	public function openuser_loginUser($provider,$theuser)
	{
		
		global $CurrentUser;

		$user = $this->getUserByOpenUserID($provider,$theuser);
		if ($user)
		{

			$this->setUserSessionInformation($user, true);
			$_SESSION['invalid_password_retry']=0;
			$url = $this->getLoginRedirectURL();
			$this->setLoginRedirectURL('');
			if (empty($url)) $this->redirectToOtherAction('',OPT_REDIRECT_HTTP); else redirectURL($url);
		}
	}
	//_______________________________________________________________________________________________________________//		
	protected function associateUserWithOtherProfile($existing_user,$provider, $hybridauth_profile)
	{
		global $SystemDB;
		
		if (is_array($existing_user) && $existing_user[$this->field_id] > 0)
		{

			$datavalues["{$provider}_user_id"] = $hybridauth_profile->identifier;
			$datavalues['date_modified'] =  date ('Y-m-d H:i:s');
			$datavalues['ip'] = $_SERVER['REMOTE_ADDR'];
			$existing_additional_info = array();
			$existing_additional_info = @unserialize($existing_user['additional_info']);
			$existing_additional_info["{$provider}_info"] = get_object_vars ($hybridauth_profile);
			$datavalues['additional_info'] = serialize($existing_additional_info);
			$sql_set_cmd = UpdateSQL($datavalues); // sql injections and stuff clear it 
			$userid = intval($_SESSION['userid']);
			$sql =  "UPDATE {$this->table_items} SET ".$sql_set_cmd." WHERE {$this->field_id} = '{$userid}';";;
			$SystemDB->query($sql);
		} else trigger_error ('Association failed as user id = 0');
	}
	//_______________________________________________________________________________________________________________//	
	protected function resolveConflictWithDifferentIdentity($existing_user, $provider, $hybridauth_profile)
	{
		global $CurrentUser;
		 
		 $current_user_name = $CurrentUser->getCurrentUserName();
		if ($this->authenticated()===true && !empty($current_user_name) )
		{
			$this->associateUserWithOtherProfile($existing_user, $provider, $hybridauth_profile);
		} else
		{
			echo \__HTML::H1('Error').P("The email address {$existing_user['email_address']} was associated with a different user ID. Please login with that credential first");
			$this->loadTemplateFile('loginpage',get_defined_vars());	
		}
	}
        
        public function logout()
        {
            require_once( "hybridauth/hybridauth/Hybrid/Auth.php" );
            require_once( "hybridauth/vendor/autoload.php" );
            \Hybrid_Auth::logoutAllProviders();
            parent::logout();
                                
        }
	//_______________________________________________________________________________________________________________//	
	public function OpenLogin($provider)
	{
		// create an instance for Hybridauth with the configuration file path as parameter
		global $SystemConfig;
		
		$enable_user_registration = $this->getConfig('bool_enable_registration');
		
		if ($provider)
		{
			$provider = alpha_numeric_only($provider);
			if ($this->getConfig("bool_{$provider}_enabled"))
			{
			
                                require_once( "hybridauth/hybridauth/Hybrid/Auth.php" );
                                require_once( "hybridauth/vendor/autoload.php" );
				$hybridauth = new \Hybrid_Auth( $this->getHybridAuthConfig() );
				$adapter = $hybridauth->authenticate( $provider );
				$user_profile = $adapter->getUserProfile();
				$existing_user = $this->getUserByOpenUserID($provider,$user_profile);
				if ($existing_user) $this->openuser_loginUser($provider,$user_profile);
				else
				{
					if (!empty($email) && $existinguser_with_different_auth = $this->getUserByEmailAddress($email))
						$this->resolveConflictWithDifferentIdentity($existinguser_with_different_auth,$provider,$user_profile);
					else
					{
                                            if (!$this->openuser_createUser($provider,$user_profile))
                                            {
                                                echo \__HTML::H1('Error').\__HTML::P("Email address field is required");	                                                 
                                            }
					}
				}
			} else echo \__HTML::H1('Error').\__HTML::P("The provider {$provider} has not been enabled");	 
		} else echo \__HTML::H1('Error').\__HTML::P('Please specify a provider');	   
	}	
	//_______________________________________________________________________________________________________________//	
	public function OpenAssoc($provider)
	{
		// create an instance for Hybridauth with the configuration file path as parameter
		global $CurrentUser;
		
		$provider = alpha_numeric_only($provider);	
		if ($this->getConfig("bool_{$provider}_enabled")==1)
		{
			// invoking getCurrentUserID is workaround for gekkocms 1.1.5 bug authenticated()
			if ($this->authenticated()  && $CurrentUser->getCurrentUserID() > 0) 
			{
			
				if ($provider)
				{
					require_once( "hybridauth/Auth.php" );
					$hybridauth = new Hybrid_Auth( $this->getHybridAuthConfig() );
					$adapter = $hybridauth->authenticate( $provider );
					$user_profile = $adapter->getUserProfile();
					$existing_user = $this->getUserByOpenUserID($provider,$user_profile);
					
					// Suggest a fix for this
					if ($existing_user)
					{
						if ($existing_user ['id'] != $CurrentUser->getCurrentUserID())
						{
							echo \__HTML::H1('Error').\__HTML::P("There is another user that already has been logged in with that identity ");	
							return false; 
						}
					}
					////////////////////////
					$this->resolveConflictWithDifferentIdentity($CurrentUser->getCurrentUserInfo(),$provider,$user_profile);
				} else echo \__HTML::H1('Error').\__HTML::P('Please specify a provider');	   
			} else 
			{
				echo \__HTML::H1('Error').\__HTML::P('Not logged in');
				$this->loadTemplateFile('loginpage',get_defined_vars());
			}
		} else 
		{
			echo \__HTML::H1('Error').\__HTML::P('This authentication is not enabled');
		}
	}	
	
	//_______________________________________________________________________________________________________________//	
	public function OpenProcess()
	{
		require_once( "hybridauth/hybridauth/Hybrid/Auth.php" );
                require_once( "hybridauth/vendor/autoload.php" );
		require_once( "hybridauth/hybridauth/Hybrid/Endpoint.php" ); 
		\Hybrid_Endpoint::process();
	}
	//_______________________________________________________________________________________________________________//	
	public function getHybridAuthConfig()
	{ 
		global $SystemConfig;
		
		$openusers_config = $this->getConfig( '',true);
		 // use gekko_config instead of $this->getconfig to reduce sql query
		$force_ssl = $this->getConfig('force_ssl_authentication');
		$process_url = $this->createFriendlyURL('action=openprocess');
		if ($force_ssl)
			$base_url = force_HTTPS_url().$process_url;
		else
			$base_url = SCHLIX_SITE_URL.$process_url;
		return array(
			"base_url" => $base_url, "debug_mode" => false,"debug_file" => '',
			"providers" => array ( 
				"Live" => array ("enabled" => true, "keys" => array ( "id" => $this->getConfig('str_live_key'), "secret" => $this->getConfig('str_live_secret') )),			
				"OpenID" => array ("enabled" => $this->getConfig('bool_openid_enabled')),
				"Facebook" => array ("enabled" => true, "keys" => array ( "id" =>  $this->getConfig('str_facebook_key'), "secret" => $this->getConfig('str_facebook_secret') ),"trustForwarded" => true ),
                            "Twitter" => array ("enabled" => true, "keys" => array ( "key" =>  $this->getConfig('str_twitter_key'), "secret" => $this->getConfig('str_twitter_secret') ),"includeEmail" => $this->getConfig('bool_twitter_include_email') ),
				"Yahoo" => array ("enabled" => $this->getConfig('bool_yahoo_enabled'), "keys" => array ( "id" => $this->getConfig('str_yahoo_key'), "secret" => $this->getConfig('str_yahoo_secret') )),
//				"AOL"  => array ("enabled" => true),
				"Google" => array ("enabled" => $this->getConfig('bool_google_enabled'), "keys"    => array ( "id" => $this->getConfig('str_google_key'), "secret" => $this->getConfig('str_google_key') ) ),
				"LinkedIn" => array ("enabled" => true,"keys" => array ( "key" => $this->getConfig('str_linkedin_key'), "secret" => $this->getConfig('str_linkedin_secret')))
			));
		
	}
	
    /**
     * Creates a friendly URL
     * @param string $str
     * @return string
     */
    public function createFriendlyURL($str) {

        if (SCHLIX_SEF_ENABLED) {
            $command_array = array();
            parse_str($str, $command_array);
            if (array_key_exists('action', $command_array) && $command_array['action'] == 'main') {
                if ($command_array['pg'])
                    $final_url.="/pg{$command_array['pg']}.html";
                else
                    $final_url.='/';
            } else
                return parent::createFriendlyURL($str);
            $final_url = SCHLIX_SITE_HTTPBASE . '/' . $this->app_name . $final_url;
        } else
            return parent::createFriendlyURL($str);
        return remove_multiple_slashes($final_url);
    }

    
    public function getValidationErrorListBeforeSaveItem($datavalues) {
        $error_list = parent::getValidationErrorListBeforeSaveItem($datavalues);
        $__category_ids_array = \SCHLIX\cmsHttpInputFilter::array_int ($datavalues, '__category_ids');
        // Verify category arrays first
        if (empty($__category_ids_array))
        {
            $error_list[] = ___('There should be at least one category');
        } else if (!$this->verifyAllCategoriesExist($__category_ids_array))
        {
            $error_list[] = ___('One or more of the specified categories are invalid');
        }
        return $error_list;
    }


    /**
     * Do something after item has been saved
     * @param array $datavalues
     * @param array $original_datavalues
     * @param array $previous_item
     * @param array $retval
     */
    public function onAfterSaveItem($datavalues, $original_datavalues, $previous_item, $retval) 
    {
        
        parent::onAfterSaveItem($datavalues, $original_datavalues, $previous_item, $retval);
        if ($retval['status'] == SAVE_OK)
        {
            // set the group
            
            $this->setItemCategories($retval['id'], array_values($original_datavalues['__category_ids']));
            $this->setItemPrimaryCategory($retval['id'], $original_datavalues['primary_cid']);
        }
    }    
    /**
     * Interprets command from SEO-friendly URL
     * @param string $urlpath
     * @return array
     */
    public function interpretFriendlyURL($urlpath) {
        if (SCHLIX_SEF_ENABLED && !fget_string('app')) {
            $parsedurl = $this->probeFriendlyURLDestination($urlpath);
            $url = $parsedurl['url'];
            $url_array = $parsedurl['url_array'];
            if ($url_array[0] == 'rss')
                $command['action'] = 'rss';
            else
            if ($c = preg_match_all("/(pg)(\d+).*?(html)/is", $url_array[0], $x)) {
                $command = array();
                $folder_requestpage = $x[2][0];
                $command['pg'] = $folder_requestpage;
                $command['action'] = 'main';
            } else
                return parent::interpretFriendlyURL($urlpath);
        } else {
            return parent::interpretFriendlyURL($urlpath);
        }
        return $command;
    }

    //_______________________________________________________________________________________________________________//
    /**
     * Returns an array containing on array of main page options. 
     * The values of the options will still be evaluated as a flat list array, 
     * however it is sectioned into array with the following keys:
     * header, value, type, and options.
     * - Label: section title (not used for any evaluation
     * - Type: checkboxgroup, dropdownlist, or none. If none, then it means there 
     *         are suboptions which contain another array of this
     * - Key: the key option. Please note that checkboxgroup doesn't have a key
     *        since the keys are in the options
     * - Options: an array with 2 keys: label and key
     * 
     * @return array
     */    
    
    public function getMainpageMetaOptionKeys() {
        return parent::getMainpageMetaOptionKeys();
    }
    //_______________________________________________________________________________________________________________//
    /**
     * Returns an array containing on array of main page options. In this base
     * class, the key is almost similar to getCategoryMetaOptionKeys
     * The values of the options will still be evaluated as a flat list array, 
     * however it is sectioned into array with the following keys:
     * header, value, type, and options.
     * - Label: section title (not used for any evaluation
     * - Type: checkboxgroup, dropdownlist, or none. If none, then it means there 
     *         are suboptions which contain another array of this
     * - Key: the key option. Please note that checkboxgroup doesn't have a key
     *        since the keys are in the options
     * - Options: an array with 2 keys: label and key
     * 
     * @return array
     */        
    public function getCategoryMetaOptionKeys() {
        return parent::getCategoryMetaOptionKeys();
    }
    
    //_______________________________________________________________________________________________________________//
    /**
     * Returns an array containing on array of item options.
     * The values of the options will still be evaluated as a flat list array, 
     * however it is sectioned into array with the following keys:
     * header, value, type, and options.
     * - Label: section title (not used for any evaluation
     * - Type: checkboxgroup, dropdownlist, or none. If none, then it means there 
     *         are suboptions which contain another array of this
     * - Key: the key option. Please note that checkboxgroup doesn't have a key
     *        since the keys are in the options
     * - Options: an array with 2 keys: label and key
     * 
     * @return array
     */
    public function getItemMetaOptionKeys() {
        return parent::getItemMetaOptionKeys();
    }


    /**
     * Returns the default category ID 
     * @return int
     */
    public function getDefaultCategoryID() {
        return 0;
    }  


    /**
     * Run command passed by the main router
     * @param array $command
     * @return bool
     */
    public function Run($command) {
        global $CurrentUser;
 		if ($this->backend_mode != 1)
		{

			if ($this->authenticated() === true && $CurrentUser->getCurrentUserID() > 0)
			{
				switch ($command['action'])
				{
					case 'openassoc': $this->OpenAssoc($command['provider']);$this->redirectToOtherAction('');break;
					case 'openprocess': $this->OpenProcess();break;					
					default: return parent::Run($command);break;
				}
			}
			else 
			{
				switch ($command['action'])
				{
					case 'openlogin':  $this->OpenLogin($command['provider']);break;
					case 'openprocess': $this->OpenProcess();break;
                                        default: return parent::Run($command);break;
				}
			}
		}
		return true;
        return RETURN_FULLPAGE;
    }
}

?>